// file vec_mat.h

#ifndef __VECTOR&MATRIX__
#define __VECTOR&MATRIX__

#include <math.h>
#include <stdlib.h>

class MVector
     {public:
      double x,y,z;

      MVector (double mx = 1,
	       double my = 1,
	       double mz = 1) {x=mx; y=my; z=mz;};

      MVector (const MVector& n)  {x=n.x; y=n.y; z=n.z;};

      double norm3d(void) {return (double)sqrt(x*x+y*y+z*z);};
      double norm2d(void) {return (double)sqrt(x*x+y*y);};

      MVector& operator = (const MVector& n) {x=n.x; y=n.y; z=n.z;
					      return *this;};

      MVector& operator = (double n) {x=y=z=n; return *this;};

      MVector& operator += (const MVector& n) {x+=n.x; y+=n.y; z+=n.z;
					       return *this;};

      MVector& operator -= (const MVector& n) {x-=n.x; y-=n.y; z-=n.z;
					       return *this;};

      MVector& operator *= (const MVector& n)
		{double prx,pry,prz;
		 prx=y*n.z-n.y*z;
		 pry=z*n.x-n.z*x;
		 prz=x*n.y-n.x*y;
		 x=prx; y=pry; z=prz;
		 return *this;};

      MVector& operator *= (double n) {x*=n; y*=n; z*=n; return *this;};

      MVector& operator /= (double n) {x/=n; y/=n; z/=n; return *this;};

      int operator < (double v) {return x<v&&y<v&&z<v;};
      int operator > (double v) {return x>v&&y>v&&z>v;};

      friend MVector operator + (const MVector& u, const MVector& v)
		     {return MVector (u.x+v.x,u.y+v.y,u.z+v.z);};
	     MVector operator - () const {return MVector(-x,-y,-z);};
      friend MVector operator - (const MVector& u, const MVector& v)
		     {return MVector (u.x-v.x,u.y-v.y,u.z-v.z);};
      friend MVector operator * (const MVector& u, const MVector& v)
		     {return MVector (u.y*v.z-u.z*v.y,
				      u.z*v.x-u.x*v.z,
				      u.x*v.y-u.y*v.x);
		     };
      friend MVector operator * (double u, const MVector& v)
		     {return MVector (u*v.x,u*v.y,u*v.z);};
      friend MVector operator * (const MVector& u, double v)
		     {return MVector (u.x*v,u.y*v,u.z*v);};
      friend double  operator & (const MVector& u, const MVector& v)
		     {return u.x*v.x+u.y*v.y+u.z*v.z;};
      friend MVector operator ^ (const MVector& u, const MVector& v)
		     {return MVector (u.x*v.x,u.y*v.y,u.z*v.z);};
      friend MVector operator / (const MVector& u, double v)
		     {return MVector (u.x/v,u.y/v,u.z/v);};
      friend MVector operator / (const MVector& u, const MVector& v)
		     {return MVector (u.x/v.x,u.y/v.y,u.z/v.z);};

     };

     MVector normalized3d(MVector& n)
	     {double n3=n.norm3d();
	      return n/n3;
	     };

     MVector normalized2d(MVector& n)
	     {double n2=n.norm2d();
	      return n/n2;
	     };

     MVector RNDMVector(void)
	     {MVector v = (rand() - 0.5*RAND_MAX,
			   rand() - 0.5*RAND_MAX,
			   rand() - 0.5*RAND_MAX
			  );
	      return normalized3d(v);
	     };

class MMatrix
     {public:
      double ar[4][4];

      MMatrix () {};
      MMatrix (const MVector&);
      MMatrix(double);
      MMatrix& operator += (const MMatrix&);
      MMatrix& operator -= (const MMatrix&);
      MMatrix& operator *= (const MMatrix&);
      MMatrix& operator *= (double);
      MMatrix& operator /= (double);

      friend MMatrix operator + (const MMatrix&, const MMatrix&);
      friend MMatrix operator - (const MMatrix&, const MMatrix&);
      friend MMatrix operator * (const MMatrix&, const MMatrix&);
      friend MMatrix operator * (double, const MMatrix&);
      friend MMatrix operator * (const MMatrix&, double);
      friend MVector operator * (const MMatrix&, const MVector&);
      friend MMatrix operator / (const MMatrix&, const MMatrix&);
      friend MMatrix operator / (const MMatrix&, double);

      void invert    (void);
      void transpose (void);

      MMatrix Translate (const MVector&);
      MMatrix Scale     (const MVector&);
      MMatrix RotateX   (double);
      MMatrix RotateY   (double);
      MMatrix RotateZ   (double);
      MMatrix MirrorX 	(void);
      MMatrix MirrorY 	(void);
      MMatrix MirrorZ 	(void);
     };

     MMatrix :: MMatrix (const MVector& v)
      {int i,j;
       for (i=0;i<4;i++)
	  for (j=0;j<4; j++) ar[i][j]=0;
       ar[0][0]=v.x; ar[1][1]=v.y;
       ar[2][2]=v.z; ar[3][3]=1.0;
      };

     MMatrix :: MMatrix (double v)
      {int i,j;
       for (i=0;i<4;i++)
	  for (j=0;j<4; j++) ar[i][j]=(i==j)?v:0.0;
       ar[3][3]=1;
      };

     MMatrix& MMatrix :: operator += (const MMatrix& v)
      {int i,j;
       for (i=0;i<4;i++)
	  for (j=0;j<4; j++) ar[i][j]+=v.ar[i][j];
       return *this;
      };

     MMatrix& MMatrix :: operator -= (const MMatrix& v)
      {int i,j;
       for (i=0;i<4;i++)
	  for (j=0;j<4; j++) ar[i][j]-=v.ar[i][j];
       return *this;
      };

     MMatrix& MMatrix :: operator *= (double v)
      {int i,j;
       for (i=0;i<4;i++)
	  for (j=0;j<4; j++) ar[i][j]*=v;
       return *this;
      };

     MMatrix& MMatrix :: operator /= (double v)
      {int i,j;
       for (i=0;i<4;i++)
	  for (j=0;j<4; j++) ar[i][j]*=v;
       return *this;
      };

     MMatrix& MMatrix :: operator *= (const MMatrix& v)
      {int i,j,k;
       MMatrix proM = *this;
       for (i=0;i<4;i++)
	  for (j=0;j<4; j++)
	   {ar[i][j]=0;
	    for (k=0; k<4; k++) ar[i][j]+=proM.ar[i][k]*v.ar[k][j];
	   };
       return *this;
      };

     void MMatrix ::invert (void)
      {MMatrix n(1);
       int i,j,k;
       double d,m;
       for (i=0; i<4;i++)
	  {d=ar[i][i];
	   if (d!=1.0)
	    {for (j=0;j<4;j++)
	     {n.ar[i][j]/=d;
	      ar[i][j]/=d;
	     };
	    };
	   for (j=0;j<4;j++)
	    {if(j!=i)
	      {if(ar[j][i]!=0.0)
		 {m=ar[j][i];
		   for(k=0; k<4; k++)
		    {ar[j][k]-=m*ar[i][k];
		     n.ar[j][k]-=m*ar[i][k];
		    };
		 };
	      };
	    };
	  };
       *this =n;
      };

     void MMatrix :: transpose()
      {double vr;
       int i,j;
       for (i=0; i<3; i++)
	for (j=i+1; j<4; j++)
	    {vr=ar[i][j];
	     ar[i][j]=ar[j][i];
	     ar[j][i]=vr;
	    };
      };

     MMatrix operator + (const MMatrix &A, const MMatrix& B)
      {MMatrix res;
       int i,j;
       for (i=0; i<4; i++)
	 for(j=0; j<4; j++)
	  {res.ar[i][j]=A.ar[i][j]+B.ar[i][j];};
       return res;
      };

     MMatrix operator - (const MMatrix &A, const MMatrix& B)
      {MMatrix res;
       int i,j;
       for (i=0; i<4; i++)
	 for(j=0; j<4; j++)
	  {res.ar[i][j]=A.ar[i][j]-B.ar[i][j];};
       return res;
      };

     MMatrix operator * (const MMatrix &A, double v)
      {MMatrix res;
       int i,j;
       for (i=0; i<4; i++)
	 for(j=0; j<4; j++)
	  {res.ar[i][j]=A.ar[i][j]*v;};
       return res;
      };

     MMatrix operator * (double v, const MMatrix &A)
      {MMatrix res;
       int i,j;
       for (i=0; i<4; i++)
	 for(j=0; j<4; j++)
	  {res.ar[i][j]=A.ar[i][j]*v;};
       return res;
      };

     MMatrix operator / (const MMatrix &A, double v)
      {MMatrix res;
       int i,j;
       for (i=0; i<4; i++)
	 for(j=0; j<4; j++)
	  {res.ar[i][j]=A.ar[i][j]/v;};
       return res;
      };

     MMatrix operator * (const MMatrix &A, const MMatrix& B)
      {int i,j,k;
       MMatrix res;
       for (i=0;i<4;i++)
	  for (j=0;j<4; j++)
	   {res.ar[i][j]=0;
	    for (k=0; k<4; k++) res.ar[i][j]+=A.ar[i][k]*B.ar[k][j];
	   };
       return res;
      };

     MVector operator * (const MMatrix& A, const MVector& v)
      {MVector res;
       double nm;
       res.x=v.x*A.ar[0][0]+v.y*A.ar[1][0]+v.z*A.ar[2][0]+A.ar[3][0];
       res.y=v.x*A.ar[0][1]+v.y*A.ar[1][1]+v.z*A.ar[2][1]+A.ar[3][1];
       res.z=v.x*A.ar[0][2]+v.y*A.ar[1][2]+v.z*A.ar[2][2]+A.ar[3][2];
       nm   =v.x*A.ar[0][3]+v.y*A.ar[1][3]+v.z*A.ar[2][3]+A.ar[3][3];
       if (nm!=1.0) res/=nm;
       return res;
      };

     MMatrix Translate (const MVector& v)
      {MMatrix res(1);
       res.ar[3][0]=v.x;
       res.ar[3][1]=v.y;
       res.ar[3][2]=v.z;
       return res;
      };

     MMatrix Scale (const MVector& v)
      {MMatrix res(v);
       return res;
      };

     MMatrix RotateX (double Angle)
      {MMatrix res(1);
       double Cosi = cos(Angle);
       double Sinu = sin(Angle);
       res.ar[1][1]=  Cosi; res.ar[1][2]= Sinu;
       res.ar[2][1]= -Sinu; res.ar[2][2]= Cosi;
       return res;
      };

     MMatrix RotateY (double Angle)
      {MMatrix res(1);
       double Cosi = cos(Angle);
       double Sinu = sin(Angle);
       res.ar[0][0]=  Cosi; res.ar[0][2]= Sinu;
       res.ar[2][0]= -Sinu; res.ar[2][2]= Cosi;
       return res;
      };

     MMatrix RotateZ (double Angle)
      {MMatrix res(1);
       double Cosi = cos(Angle);
       double Sinu = sin(Angle);
       res.ar[0][0]=  Cosi; res.ar[0][1]= Sinu;
       res.ar[1][0]= -Sinu; res.ar[1][1]= Cosi;
       return res;
      };

MMatrix	Rotation ( const MVector& axis, double angle )
{
	MMatrix	res ( 1 );
	double  Cosine = cos ( angle );
	double  Sine   = sin ( angle );

	res.ar[0][0] = axis.x * axis.x + ( 1 - axis.x * axis.x ) * Cosine;
	res.ar[0][1] = axis.x * axis.y * ( 1 - Cosine ) + axis.z * Sine;
	res.ar[0][2] = axis.x * axis.z * ( 1 - Cosine ) - axis.y * Sine;
	res.ar[0][3] = 0;

	res.ar[1][0] = axis.x * axis.y * ( 1 - Cosine ) - axis.z * Sine;
	res.ar[1][1] = axis.y * axis.y + ( 1 - axis.y * axis.y ) * Cosine;
	res.ar[1][2] = axis.y * axis.z * ( 1 - Cosine ) + axis.x * Sine;
	res.ar[1][3] = 0;

	res.ar[2][0] = axis.x * axis.z * ( 1 - Cosine ) + axis.y * Sine;
	res.ar[2][1] = axis.y * axis.z * ( 1 - Cosine ) - axis.x * Sine;
	res.ar[2][2] = axis.z * axis.z + ( 1 - axis.z * axis.z ) * Cosine;
	res.ar[2][3] = 0;

	res.ar[3][0] = 0;
	res.ar[3][1] = 0;
	res.ar[3][2] = 0;
	res.ar[3][3] = 1;

	return res;
}

     MMatrix MirrorX (void)
      {MMatrix res(1); res.ar[0][0]=-1; return res;};

     MMatrix MirrorY (void)
      {MMatrix res(1); res.ar[1][1]=-1; return res;};

     MMatrix MirrorZ (void)
      {MMatrix res(1); res.ar[2][2]=-1; return res;};

#endif